package com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.service;

import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.service.CrudService;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.entity.ReserveElectricityWork;
import com.jeeplus.modules.cg.reservemanagementcenter.reserveManage.mapper.ReserveElectricityWorkMapper;
import com.jeeplus.modules.cg.reservemanagementcenter.statisticalTable.entity.History;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

/**
 * 储备项目带电作业信息 service
 * @author: 王强
 * @create: 2022-07-22 09:10
 **/
@Service
@Transactional(readOnly = true)
public class ReserveElectricityWorkService extends CrudService<ReserveElectricityWorkMapper, ReserveElectricityWork> {

    @Autowired
    private ReserveElectricityWorkMapper dao;

    /**
     * 修改用户分界开关数量
     * @param allNum
     */
    @Transactional(readOnly = false)
    public void updateSwitchNum(String allNum){
        dao.updateSwitchNum(allNum);
    }

    /**
     * 获取所有的  项目储备编号
     * @return
     */
    public List<String> getAllProId(){
        return dao.getAllProId();
    }

    /**
     * 获取所有的  项目储备编号
     * @return
     */
    public List<String> getAllProId2(){
        return dao.getAllProId2();
    }

    /**
     * 查询已有的参数数据 并分类返回已存在数据和未存在数据
     * @param reserveManagement
     * @return
     */
    public Map<String, List<ReserveElectricityWork>> getExistProjectList(List<ReserveElectricityWork> reserveManagement){
        Map<String,List<ReserveElectricityWork>> map = new HashMap();
        List<ReserveElectricityWork> existReserveManagementList = new ArrayList<>();
        Iterator<ReserveElectricityWork> iterator = reserveManagement.iterator();
        while (iterator.hasNext()) {
            ReserveElectricityWork rm = iterator.next();
            //对数据进行处理
            rm.preInsert();
            ReserveElectricityWork reserveManag = dao.queryByProjectReserveId(rm);

            //数据库中已含有的数据在list中删除并创建一个list加入 区分为已存在数据
            if (null != reserveManag) {
                rm.setId(reserveManag.getId());
                existReserveManagementList.add(rm);
                iterator.remove();//使用迭代器删除已存在的数据
            }

        }
        map.put("without",reserveManagement);
        map.put("exist",existReserveManagementList);
        return map;
    }

    /**
     * 批量添加数据
     * @param reserveManagementList
     */
    @Transactional(readOnly = false)
    public void insertList(List<ReserveElectricityWork> reserveManagementList){

        dao.insertList(reserveManagementList);
    }

    /**
     * 专职修改方法
     * @param reserveManagement
     */
    @Transactional(readOnly = false)
    public void update(ReserveElectricityWork reserveManagement) {

        super.save(reserveManagement);
    }

    public Page<ReserveElectricityWork> findPage(Page<ReserveElectricityWork> page, ReserveElectricityWork reserveManagement) {
        Page<ReserveElectricityWork> pageList = super.findPage(page, reserveManagement);
        return pageList;
    }

    public List<ReserveElectricityWork> exportFindList( ReserveElectricityWork reserveManagement) {
        List<ReserveElectricityWork> pageList = dao.exportFindList(reserveManagement);
        return pageList;
    }

    /**
     * 新增方法
     * @param entity
     */
    @Transactional(readOnly = false)
    public void save(ReserveElectricityWork entity) {
        //查询数据表中是否含有同项目储备编号的数据 若有则进行修改 没有则新增
        //根据项目储备编号 查询数据
        if (entity.getIsNewRecord()){
            entity.preInsert();
            mapper.insert(entity);
        }else{
            entity.preUpdate();
            mapper.update(entity);
        }
    }

    public String getByProjectReserveId(String projectReserveId){
        return dao.getByProjectReserveId(projectReserveId);
    }

    public List<History> getHistoryByExecuteId(String executeId){
        return dao.getHistoryByExecuteId(executeId);
    }

    public String getByExecuteId(String executeId){
        return dao.getByExecuteId(executeId);
    }
}
