package com.jeeplus.modules.sg.balancedlibrary.transitMaterialGoods.service;

import com.jeeplus.common.utils.StringUtils;
import com.jeeplus.core.persistence.Page;
import com.jeeplus.core.service.CrudService;
import com.jeeplus.modules.sg.balancedlibrary.reportPerson.entity.ReportPerson;
import com.jeeplus.modules.sg.balancedlibrary.reportPerson.mapper.ReportPersonMapper;
import com.jeeplus.modules.sg.balancedlibrary.transitMaterialGoods.entity.MaterialGoodsInfo;
import com.jeeplus.modules.sg.balancedlibrary.transitMaterialGoods.entity.TransitMaterialGoods;
import com.jeeplus.modules.sg.balancedlibrary.transitMaterialGoods.mapper.TransitMaterialGoodsMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 在途物资Service
 * @author: 王强
 * @create: 2022-08-30 14:12
 **/
@Service
@Transactional(readOnly = true)
public class TransitMaterialGoodsService extends CrudService<TransitMaterialGoodsMapper, TransitMaterialGoods> {

    @Autowired
    private ReportPersonMapper reportPersonMapper;

    @Autowired
    private TransitMaterialGoodsMapper mapper;

    public void comparativeData(List<TransitMaterialGoods> transitMaterialGoodsList, List<MaterialGoodsInfo> materialGoodsInfoList) {

        //在途物资统计表 key=统计物料编码-部门 去重汇总统计
        Map<String, BigDecimal> onPassageMaterialsMap = new HashMap<>();
        //项目物资需求表 key=统计物料编码-部门 去重汇总统计
        Map<String, BigDecimal> materialInformationMap = new HashMap<>();

        //在途物资统计表 如果计划编制人为空存储
        //Map<String, BigDecimal> isEmptyMap = new HashMap<>();

        //在途物资统计表与项目物资需求表比较  在途物资统计表J列>=项目物资需求表H列 标识
        List<String> flagList = new ArrayList<>();
        /**************************************************************************/


        //从数据库中获取所有的提报人与部门
        Map<String, String> reportPersonMap = getReportPerson();

        //在途物资统计表 去重汇总统计
        transitMaterialGoodsList.forEach(opm -> {
            if (StringUtils.isBlank(opm.getContractCount())) {
                opm.setContractCount("0");
            }
            //根据提报人 找到对应部门
            String reportPersonOfDepartment = reportPersonMap.get(opm.getPlanner());
            //如果没有部门 表示空部门 物料编码-部门
            String key = "";
            if (reportPersonOfDepartment == null) {
                key = opm.getMaterialCode() + "-";
            } else {
                key = opm.getMaterialCode() + "-" + reportPersonOfDepartment;
            }
            if (onPassageMaterialsMap.containsKey(key)) {
                onPassageMaterialsMap.put(key, new BigDecimal(opm.getContractCount()).add(onPassageMaterialsMap.get(key)));
            } else {
                if (StringUtils.isNotBlank(opm.getMaterialCode())) {
                    onPassageMaterialsMap.put(key, new BigDecimal(opm.getContractCount()));
                }
            }
        });

        //项目物资需求表 去重汇总统计
        materialGoodsInfoList.forEach(m -> {
            if (StringUtils.isBlank(m.getTotal())) {
                m.setTotal("0");
            }
            //物料编码-部门
            String key = m.getMaterialCode() + "-" + m.getReportingDepartment();
            if (materialInformationMap.containsKey(key)) {
                materialInformationMap.put(key, new BigDecimal(m.getTotal()).add(materialInformationMap.get(key)));
            } else {
                if (StringUtils.isNotBlank(m.getMaterialCode())) {
                    materialInformationMap.put(key, new BigDecimal(m.getTotal()));
                }
            }
        });

        //两表比较 添加标识
        materialInformationMap.forEach((k, v) -> {
            if (onPassageMaterialsMap.containsKey(k)) {
                if (onPassageMaterialsMap.get(k).compareTo(v) >= 0) {
                    flagList.add(k);
                }
            }
        });

        //根据标识在原表中追加备注
        materialGoodsInfoList.forEach(l -> {
            String key = l.getMaterialCode() + "-" + l.getReportingDepartment();
            if (flagList.contains(key)) {
                l.setRemarkText("在途已有" + onPassageMaterialsMap.get(key) + "单位，请核实本次需求");
            }
        });
    }

    /**
     * 查询数据库获取对应的部门及部门下的人
     *
     * @return
     */
    public Map<String, String> getReportPerson() {
        Map<String, String> map = new HashMap<>();
        List<ReportPerson> departmentGroupByPerson = reportPersonMapper.findDepartmentGroupByPerson();
        departmentGroupByPerson.forEach(l -> {
            map.put(l.getReportPerson(), l.getReportDepartment());
        });
        return map;
    }

    @Transactional(readOnly = false)
    public void insertList(List<TransitMaterialGoods> list) {
        for (TransitMaterialGoods goods : list){
            goods.preInsert();
        }
        //导入前先清空在途物资数据
        mapper.deleteZTList();
        mapper.insertList(list);
    }

    @Override
    public Page<TransitMaterialGoods> findPage(Page<TransitMaterialGoods> page, TransitMaterialGoods entity) {
        return super.findPage(page, entity);
    }

    /**
     * 批量删除在途物资
     */
    @Transactional(readOnly = false)
    public void deleteList(List<String> list) {
        mapper.delList(list);
    }

    public List<TransitMaterialGoods> getAllGoods(){
        return mapper.getAllGoods();
    }
}
